using System;
using UnityEngine;
using UnityEngine.Events;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace RightClick
{
    public class rightClickOptions
    {
        public static void OnLoad()
        {
            //show ad informing about update
            //using key "QROM-Update-v2.0"
            if (!PlayerPrefs.HasKey("QROM-Update-v2.0"))
            {
                DialogBoxManager.Dialog("=================== Quick Right Click Options Update ===================\nThe newest update for Quick Right Click Options adds a variety of optimizations and\ncode restructuring to improve FPS, alongside new, bindable keys for each function!\nYou can find them in Settings > Controls > (scroll down) Modded Keys.\nFor more information, check the mod's steam page.", new DialogButton("Go Away!", true, () => PlayerPrefs.SetInt("QROM-Update-v2.0", 1)));
            }

            //upon loading, re-register keybinds (in case set)
            initBinds();
        }

        public static void Main()
        {            
            ModAPI.OnItemSpawned += (sender, args) =>
            {
                //if new item has personbehaviour (is alive), then add rightclickbehaviour to them
                //safecheck as well (not already rightclickbehaviour, incase of loading/copypasting/etc)
                var PB = args.Instance.gameObject.GetComponent<PersonBehaviour>();
                if (PB && !PB.gameObject.GetComponent<RightClickPersonBehaviour>()) {
                    PB.gameObject.AddComponent<RightClickPersonBehaviour>();
                } else if (PB && PB.gameObject.GetComponent<RightClickPersonBehaviour>()) {
                    //destroy now defunct rcbehaviours in case of objects with older code
                    //shitty methodology, will be deleted later
                    var detected = false;
                    foreach (var limb in PB.Limbs)
                    {
                        //if old rclimbbehaviour
                        if (limb.gameObject.GetComponent<RightClickLimbBehaviour>())
                        {
                            detected = true;
                            //attempt to destroy limbbehaviour immediately
                            GameObject.DestroyImmediate(limb.gameObject.GetComponent<RightClickLimbBehaviour>());

                            //if behaviours still exist, log
                            if (limb.gameObject.GetComponent<RightClickLimbBehaviour>())
                            {
                                Debug.LogWarning("Defunct RightClickLimbBehaviour could not be deleted.");
                            }
                        }
                    }

                    //attempt to destroy personbehaviour immediately, if old
                    //then re-add updated if deletion successful
                    if (detected)
                    {
                        Debug.LogWarning("Defunct RightClickPersonBehaviour detected, replacing...");
                        GameObject.DestroyImmediate(PB.gameObject.GetComponent<RightClickPersonBehaviour>());
                        if (!PB.gameObject.GetComponent<RightClickPersonBehaviour>()) {
                            PB.gameObject.AddComponent<RightClickPersonBehaviour>();
                        }
                    }
                }
            };

            //initialize key binds
            //first register keymonitor class, then call init bind functions to set defaults
            ModAPI.Register<KeyMonitor>();
            initBinds();
        }

        private const string inputKOKey = "RightClickOptions-KO";
        private const string inputWakeKey = "RightClickOptions-Wake";
        private const string inputKillKey = "RightClickOptions-Kill";
        private const string inputReviveKey = "RightClickOptions-Revive";
        private const string inputReanimKey = "RightClickOptions-Reanimate";
        private const string inputImplodeKey = "RightClickOptions-Implode";
        private const string inputZombifyKey = "RightClickOptions-Zombify";

        private static void initBinds()
        {
            if (!InputSystem.Has(inputKOKey)) ModAPI.RegisterInput("[RCO] Knockout Keybind", inputKOKey, KeyCode.None);
            if (!InputSystem.Has(inputWakeKey)) ModAPI.RegisterInput("[RCO] Awaken Keybind", inputWakeKey, KeyCode.None);
            if (!InputSystem.Has(inputKillKey)) ModAPI.RegisterInput("[RCO] Kill Keybind", inputKillKey, KeyCode.None);
            if (!InputSystem.Has(inputReviveKey)) ModAPI.RegisterInput("[RCO] Revive Keybind", inputReviveKey, KeyCode.None);
            if (!InputSystem.Has(inputReanimKey)) ModAPI.RegisterInput("[RCO] Reanimate Keybind", inputReanimKey, KeyCode.None);
            if (!InputSystem.Has(inputImplodeKey)) ModAPI.RegisterInput("[RCO] Implode Keybind", inputImplodeKey, KeyCode.None);
            if (!InputSystem.Has(inputZombifyKey)) ModAPI.RegisterInput("[RCO] Zombify Keybind", inputZombifyKey, KeyCode.None);
        }

        public class KeyMonitor : MonoBehaviour
        {
            private void Update()
            {
                //if not editing and something is selected
                if (DialogBox.IsAnyDialogboxOpen || Global.ActiveUiBlock || TriggerEditorBehaviour.IsBeingEdited || SelectionController.Main.SelectedObjects.Count == 0) return;
                
                //if input keys are down
                if (InputSystem.Down(inputKOKey)) {
                    //iterate through selected objects 
                    //note SelectionController returns physicalbehaviour, get gameobject to derive other behaviours
                    foreach (var selectedObj in SelectionController.Main.SelectedObjects)
                    {
                        //detect if alive (via limbobject)
                        if (selectedObj.gameObject.GetComponent<LimbBehaviour>())
                        {
                            RightClick.rightClickEffects.knockOut(selectedObj.gameObject.GetComponent<LimbBehaviour>().Person);
                        }
                    }
                }
                if (InputSystem.Down(inputWakeKey))
                {
                    foreach (var selectedObj in SelectionController.Main.SelectedObjects)
                    {
                        if (selectedObj.gameObject.GetComponent<LimbBehaviour>())
                        {
                            RightClick.rightClickEffects.awaken(selectedObj.gameObject.GetComponent<LimbBehaviour>().Person);
                        }
                    }
                }
                if (InputSystem.Down(inputKillKey)) {
                    foreach (var selectedObj in SelectionController.Main.SelectedObjects)
                    {
                        if (selectedObj.gameObject.GetComponent<LimbBehaviour>()) {
                            RightClick.rightClickEffects.kill(selectedObj.gameObject.GetComponent<LimbBehaviour>().Person);
                        }
                    }
                }
                if (InputSystem.Down(inputReviveKey))
                {
                    foreach (var selectedObj in SelectionController.Main.SelectedObjects)
                    {
                        if (selectedObj.gameObject.GetComponent<LimbBehaviour>())
                        {
                            RightClick.rightClickEffects.revive(selectedObj.gameObject.GetComponent<LimbBehaviour>().Person);
                        }
                    }
                }
                if (InputSystem.Down(inputReanimKey))
                {
                    foreach (var selectedObj in SelectionController.Main.SelectedObjects)
                    {
                        if (selectedObj.gameObject.GetComponent<LimbBehaviour>())
                        {
                            RightClick.rightClickEffects.reanimate(selectedObj.gameObject.GetComponent<LimbBehaviour>().Person);
                        }
                    }
                }
                if (InputSystem.Down(inputImplodeKey))
                {
                    foreach (var selectedObj in SelectionController.Main.SelectedObjects)
                    {
                        if (selectedObj.gameObject.GetComponent<LimbBehaviour>())
                        {
                            RightClick.rightClickEffects.implode(selectedObj.gameObject.GetComponent<LimbBehaviour>().Person);
                        }
                    }
                }
                if (InputSystem.Down(inputZombifyKey))
                {
                    foreach (var selectedObj in SelectionController.Main.SelectedObjects)
                    {
                        if (selectedObj.gameObject.GetComponent<LimbBehaviour>())
                        {
                            RightClick.rightClickEffects.zombify(selectedObj.gameObject.GetComponent<LimbBehaviour>().Person);
                        }
                    }
                }
            }
        }
    }
}
// Originally uploaded by 'Pump3d'. Do not reupload without their explicit permission.
