using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace RightClick
{
    public static class rightClickEffects {
        public static SyringeBehaviour lifeS = new LifeSyringe();
        public static SyringeBehaviour zombieS = new ZombieSyringe();
        public static Vector2 limits = new LifeSyringe().Limits;
        
        public static void knockOut(PersonBehaviour PB) {
            PB.Consciousness = 0f;
            foreach (var limb in PB.Limbs)
            {
                limb.Numbness = 1f;
            }
        }

        public static void awaken(PersonBehaviour PB)
        {
            PB.OxygenLevel = 1f;
            PB.Consciousness = 1f;
            PB.ShockLevel = 0f;
            PB.PainLevel = 0f;
            PB.AdrenalineLevel = 0f;

            foreach (var limb in PB.Limbs)
            {
                limb.Numbness = 0f;
                limb.CirculationBehaviour.InternalBleedingIntensity = 0f;
            }
        }

        public static void kill(PersonBehaviour PB)
        {
            //turn off invincibility in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().invincible = false;
            
            foreach (var limb in PB.Limbs)
            {
                limb.CirculationBehaviour.IsPump = false;
                limb.CirculationBehaviour.ForceSetAllLiquid(0f);
            }
        }

        public static void revive(PersonBehaviour PB)
        {
            foreach (var limb in PB.Limbs)
            {
                if (limb.IsAndroid)
                {
                    CirculationBehaviour circulationBehaviour = limb.GetComponent<CirculationBehaviour>();
                    limb.Health = limb.InitialHealth;
                    limb.HealBone();
                    circulationBehaviour.HealBleeding();
                    circulationBehaviour.ClearLiquid();
                    circulationBehaviour.AddLiquid(limb.GetOriginalBloodType(), 1f);
                    circulationBehaviour.BloodFlow = 1f;
                    circulationBehaviour.IsPump = circulationBehaviour.WasInitiallyPumping;
                    limb.Person.Consciousness = 1f;
                    limb.PhysicalBehaviour.BurnProgress *= 0f;

                    continue;
                }

                limb.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(lifeS.GetLiquidID()), limits.y);
                //Liquid.GetLiquid(lifeS.GetLiquidID()).OnEnterLimb(limb);
                //limb.CirculationBehaviour.RemoveLiquid(Liquid.GetLiquid(lifeS.GetLiquidID()), limits.y);
            }
        }

        public static void reanimate(PersonBehaviour PB)
        {
            foreach (var limb in PB.Limbs)
            {
                limb.Person.Braindead = false;
                limb.Person.BrainDamaged = false;


                CirculationBehaviour circulationBehaviour = limb.GetComponent<CirculationBehaviour>();
                limb.HealBone();
                limb.Health = limb.InitialHealth;
                limb.Numbness = 0f;
                circulationBehaviour.HealBleeding();
                circulationBehaviour.IsPump = circulationBehaviour.WasInitiallyPumping;
                circulationBehaviour.BloodFlow = 1f;
                circulationBehaviour.ClearLiquid();
                circulationBehaviour.AddLiquid(limb.GetOriginalBloodType(), 1f);
                limb.BruiseCount = 0;
                circulationBehaviour.GunshotWoundCount = 0;
                circulationBehaviour.StabWoundCount = 0;

                limb.LungsPunctured = false;

                if (limb.HasBrain)
                {
                    limb.Person.Consciousness = 1f;
                    limb.Person.ShockLevel = 0f;
                    limb.Person.PainLevel = 0f;
                    limb.Person.OxygenLevel = 1f;
                    limb.Person.AdrenalineLevel = 1f;
                }
            }
        }

        public static void implode(PersonBehaviour PB)
        {
            //toggle imploded state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().imploded = true;

            foreach (var limb in PB.Limbs)
            {
                limb.Health = 0f;
                limb.CirculationBehaviour.IsPump = false;
                //limb.CirculationBehaviour.ForceSetAllLiquid(0f);
                limb.BreakingThreshold = 0f;
            }
        }

        public static void zombify(PersonBehaviour PB)
        {
            //toggle zombie state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().zombie = true;

            foreach (var limb in PB.Limbs)
            {
                limb.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(zombieS.GetLiquidID()), limits.y);
                Liquid.GetLiquid(zombieS.GetLiquidID()).OnEnterLimb(limb);
                limb.CirculationBehaviour.RemoveLiquid(Liquid.GetLiquid(zombieS.GetLiquidID()), limits.y);
            }
        }

        //enables and disables MUST be distinct functions rather than toggles
        //bc when multiple human(s)/parts are selected, fires multiple times
        public static void disablePain(PersonBehaviour PB)
        {
            //toggle noPain state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().noPain = true;
        }

        public static void enablePain(PersonBehaviour PB)
        {
            //toggle noPain state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().noPain = false;
        }


        public static void disableDrowning(PersonBehaviour PB)
        {
            //toggle noDrown state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().noDrown = true;
        }

        public static void enableDrowning(PersonBehaviour PB)
        {
            //toggle noDrown state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().noDrown = false;
        }


        public static void disableBurning(PersonBehaviour PB)
        {
            //toggle noBurning state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().noBurning = true;

            foreach (var limb in PB.Limbs)
            {
                var phys = limb.GetComponent<PhysicalBehaviour>();
                var properties = phys.Properties.ShallowClone();
                properties.Flammability = 0f;
                phys.Properties = properties;
            }
        }

        public static void enableBurning(PersonBehaviour PB)
        {
            //toggle noBurning state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().noBurning = false;

            foreach (var limb in PB.Limbs)
            {
                var phys = limb.GetComponent<PhysicalBehaviour>();
                var properties = phys.Properties.ShallowClone();
                properties.Flammability = 1f;
                phys.Properties = properties;
            }
        }


        public static void enableSuffocation(PersonBehaviour PB)
        {
            //toggle suffocate state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().suffocate = true;
        }

        public static void disableSuffocation(PersonBehaviour PB)
        {
            //toggle suffocate state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().suffocate = false;
        }


        public static void enableInvincibility(PersonBehaviour PB)
        {
            //toggle invincible state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().invincible = true;
        }

        public static void disableInvincibility(PersonBehaviour PB)
        {
            //toggle invincible state in modded pBehaviour
            PB.gameObject.GetComponent<RightClickPersonBehaviour>().invincible = false;
        }
    }
}
// Originally uploaded by 'Pump3d'. Do not reupload without their explicit permission.
