using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace RightClick
{
    public class RightClickPersonBehaviour : MonoBehaviour
    {
        public bool noPain = false;
        public bool noDrown = false;
        public bool noBurning = false;
        public bool invincible = false;
        public bool suffocate = false;
        public bool imploded = false;
        public bool zombie = false;
        private PersonBehaviour PB;

        void Start()
        {
            PB = gameObject.GetComponent<PersonBehaviour>();

            //buttons initialization
            //each button concept references a function in rightclickeffects as unityevent
            var knockOut = new ContextMenuButton(() => PB.Consciousness == 1f, "knockOut", "Knock Out", "KO the selected human", (UnityAction)(() => RightClick.rightClickEffects.knockOut(PB)));
            knockOut.LabelWhenMultipleAreSelected = "Knock Out";

            var awaken = new ContextMenuButton(() => PB.Consciousness != 1f && PB.IsAlive(), "wakeUp", "Wake Up", "Awaken the selected human", (UnityAction)(() => RightClick.rightClickEffects.awaken(PB)));
            awaken.LabelWhenMultipleAreSelected = "Wake up";

            var kill = new ContextMenuButton(() => PB.IsAlive(), "kill", "Kill", "Kill the selected human", (UnityAction)(() => RightClick.rightClickEffects.kill(PB)));
            kill.LabelWhenMultipleAreSelected = "Kill";

            var revive = new ContextMenuButton(() => invincible == false, "revive", "Revive", "Revive the selected human", (UnityAction)(() => RightClick.rightClickEffects.revive(PB)));
            revive.LabelWhenMultipleAreSelected = "Revive";

            var reanimate = new ContextMenuButton(() => invincible == false, "reanimate", "Reanimate", "Reanimate the selected human (only internal)", (UnityAction)(() => RightClick.rightClickEffects.reanimate(PB)));
            reanimate.LabelWhenMultipleAreSelected = "Reanimate";

            var implode = new ContextMenuButton(() => imploded == false, "implode", "Implode", "Implode the selected human", (UnityAction)(() => RightClick.rightClickEffects.implode(PB)));
            implode.LabelWhenMultipleAreSelected = "Implode";

            var zombify = new ContextMenuButton(() => zombie == false, "zombify", "Zombify", "Zombify the selected human", (UnityAction)(() => RightClick.rightClickEffects.zombify(PB)));
            zombify.LabelWhenMultipleAreSelected = "Zombify";


            var disablePain = new ContextMenuButton(() => PB.IsAlive() && noPain == false, "disablePain", "Disable Pain", "Disable the pain of the selected human", (UnityAction)(() => RightClick.rightClickEffects.disablePain(PB)));
            disablePain.LabelWhenMultipleAreSelected = "Disable Pain";

            var enablePain = new ContextMenuButton(() => PB.IsAlive() && noPain == true, "enablePain", "Enable Pain", "Enable the pain of the selected human", (UnityAction)(() => RightClick.rightClickEffects.enablePain(PB)));
            enablePain.LabelWhenMultipleAreSelected = "Enable Pain";



            var disableDrowning = new ContextMenuButton(() => PB.IsAlive() && noDrown == false && suffocate == false, "disableDrowning", "Disable Drowning", "Disable drowning of the selected human", (UnityAction)(() => RightClick.rightClickEffects.disableDrowning(PB)));
            disableDrowning.LabelWhenMultipleAreSelected = "Disable Drowning";

            var enableDrowning = new ContextMenuButton(() => PB.IsAlive() && noDrown == true && suffocate == false, "enableDrowning", "Enable Drowning", "Enable drowning of the selected human", (UnityAction)(() => RightClick.rightClickEffects.enableDrowning(PB)));
            enableDrowning.LabelWhenMultipleAreSelected = "Enable Drowning";



            var disableBurning = new ContextMenuButton(() => noBurning == false, "disableBurning", "Disable Burning", "Disable burning of the selected human", (UnityAction)(() => RightClick.rightClickEffects.disableBurning(PB)));
            disableBurning.LabelWhenMultipleAreSelected = "Disable Burning";

            var enableBurning = new ContextMenuButton(() => noBurning == true, "enableBurning", "Enable Burning", "Enable burning of the selected human", (UnityAction)(() => RightClick.rightClickEffects.enableBurning(PB)));
            enableBurning.LabelWhenMultipleAreSelected = "Enable Burning";



            var enableSuffocation = new ContextMenuButton(() => PB.IsAlive() && noDrown == false && suffocate == false, "suffocate", "Suffocate", "Suffocate the selected human", (UnityAction)(() => RightClick.rightClickEffects.enableSuffocation(PB)));
            enableSuffocation.LabelWhenMultipleAreSelected = "Suffocate";

            var disableSuffocation = new ContextMenuButton(() => PB.IsAlive() && noDrown == false && suffocate == true, "unSuffocate", "Unsuffocate", "Unsuffocate the selected human", (UnityAction)(() => RightClick.rightClickEffects.disableSuffocation(PB)));
            disableSuffocation.LabelWhenMultipleAreSelected = "Unsuffocate";



            var enableInvincibility = new ContextMenuButton(() => invincible == false, "invincible", "Make Invincible", "Make the selected human invincible", (UnityAction)(() => RightClick.rightClickEffects.enableInvincibility(PB)));
            enableInvincibility.LabelWhenMultipleAreSelected = "Make Invincible";

            var disableInvincibility = new ContextMenuButton(() => invincible == true, "unInvincible", "Remove Invincibility", "Remove the invincibility of the selected human", (UnityAction)(() => RightClick.rightClickEffects.disableInvincibility(PB)));
            disableInvincibility.LabelWhenMultipleAreSelected = "Remove Invincibility";


            foreach (var limb in PB.Limbs)
            {
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(knockOut);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(awaken);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(kill);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(revive);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(reanimate);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(implode);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(zombify);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(disablePain);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(enablePain);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(disableDrowning);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(enableDrowning);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(disableBurning);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(enableBurning);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(enableSuffocation);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(disableSuffocation);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(enableInvincibility);
                limb.PhysicalBehaviour.ContextMenuOptions.Buttons.Add(disableInvincibility);
            }
        }

        void Update()
        {
            if (noPain) {
                PB.PainLevel = 0.0f;
                PB.ShockLevel = 0.0f;
            }

            if (noDrown) {
                PB.OxygenLevel = 1f;
            }

            if (suffocate) {
                PB.OxygenLevel = 0f;
                PB.PainLevel = 1f;
            }

            //every other 3 updates bc expensive functions
            //running every frame usually causes some fps drop
            if (Time.frameCount % 3 != 0)
            {
                return;
            }

            if (noBurning) {
                foreach (var limb in PB.Limbs) {
                    CirculationBehaviour circulationBehaviour = limb.CirculationBehaviour;
                    circulationBehaviour.ClearLiquid();
                    circulationBehaviour.AddLiquid(limb.GetOriginalBloodType(), 1f);
                    circulationBehaviour.BloodFlow = 1f;
                    
                    limb.PhysicalBehaviour.Extinguish();
                    limb.PhysicalBehaviour.BurnProgress *= 0f;        
                    limb.InternalTemperature = limb.BodyTemperature;
                    limb.GetComponent<PhysicalBehaviour>().BurnProgress = 0f;
                    limb.GetComponent<SkinMaterialHandler>().RottenProgress = 0f;
                }
            }

            if (invincible) {
                foreach (var limb in PB.Limbs)
                {
                    if (limb.IsAndroid) {
                        CirculationBehaviour circulationBehaviour = limb.CirculationBehaviour;
                        limb.Health = limb.InitialHealth;
                        limb.HealBone();
                        circulationBehaviour.HealBleeding();
                        circulationBehaviour.ClearLiquid();
                        circulationBehaviour.AddLiquid(limb.GetOriginalBloodType(), 1f);
                        circulationBehaviour.BloodFlow = 1f;
                        circulationBehaviour.IsPump = circulationBehaviour.WasInitiallyPumping;
                        limb.Person.Consciousness = 1f;
                        limb.PhysicalBehaviour.BurnProgress *= 0f;        
                        continue;
                    }

                    if (limb.HasBrain) {
                        //set color of life syringe to entity's blood color
                        //so fucking garbage but whatever
                        //one wraparound would be to clone lifesyringe, but still wouldn't work cross-species
                        Liquid.GetLiquid(RightClick.rightClickEffects.lifeS.GetLiquidID()).Color = limb.GetOriginalBloodType().Color;
                        limb.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(RightClick.rightClickEffects.lifeS.GetLiquidID()), RightClick.rightClickEffects.limits.y);
                    }
                }
            }
        }
    }
}
// Originally uploaded by 'Pump3d'. Do not reupload without their explicit permission.
